from __future__ import absolute_import, unicode_literals

# karger: mostly paywalled, but sometimes...
# http://www.karger.com/Article/Pdf/351538
#
# {kid} comes from the final nonzero numbers of the article's DOI.
karger_format = 'http://www.karger.com/Article/Pdf/{kid}'

karger_journals = [
        'Acta Cytol',
        'Acta Genet Stat Med',
        'Acta Haematol',
        'Adv Otorhinolaryngol',
        'Am J Nephrol',
        'AMA Arch Ophthalmol',
        'Ann Nutr Metab',
        'Audiol Neurootol',
        'Biol Neonate',
        'Blood Purif',
        'Brain Behav Evol',
        'Cardiology',
        'Caries Res',
        'Cell Physiol Biochem',
        'Cells Tissues Organs (Print)',
        'Cerebrovasc Dis',
        'Chemotherapy',
        'Community Genet',
        'Contrib Nephrol',
        'Curr Probl Dermatol',
        'Cytogenet Cell Genet',
        'Cytogenet Genome Res',
        'Dement Geriatr Cogn Disord',
        'Dermatology (Basel)',
        'Dev Biol (Basel)',
        'Dev Neurosci',
        'Dev Ophthalmol',
        'Dig Dis',
        'Dig Surg',
        'Digestion',
        'Endocr Dev',
        'Eur Addict Res',
        'Eur Neurol',
        'Eur Surg Res',
        'Exp Cell Biol',
        'Fetal Diagn Ther',
        'Gerontology',
        'Gynecol Obstet Invest',
        'Haemostasis',
        'Horm Res',
        'Horm Res Paediatr',
        'Hum Hered',
        'Int Arch Allergy Appl Immunol',
        'Int Arch Allergy Immunol',
        'Intervirology',
        'J Biomed Sci',
        'J Innate Immun',
        'J Mol Microbiol Biotechnol',
        'J Nutrigenet Nutrigenomics',
        'J Vasc Res',
        'Kidney Blood Press Res',
        'Med Princ Pract',
        'Neonatology',
        'Nephron',
        'Nephron Clin Pract',
        'Nephron Exp Nephrol',
        'Nephron Physiol',
        'Nestle Nutr Workshop Ser Pediatr Program',
        'Neurodegener Dis',
        'Neuroendocrinology',
        'Neuroepidemiology',
        'Neuroimmunomodulation',
        'Neuropsychobiology',
        'Neurosignals',
        'ORL J Otorhinolaryngol Relat Spec',
        'Obes Facts',
        'Oncology',
        'Onkologie',
        'Ophthalmic Res',
        'Ophthalmologica',
        'Pathobiology',
        'Pathophysiol Haemost Thromb',
        'Pediatr Neurosurg',
        'Pharmacology',
        'Psychother Psychosom',
        'Public Health Genomics',
        'Respiration',
        'Sex Dev',
        'Urol Int',
        'Verh Dtsch Ges Pathol',
        'Vestn Akad Med Nauk SSSR',
        'Zhonghua Er Ke Za Zhi',
        'Zhonghua Yan Ke Za Zhi',
]
