from __future__ import absolute_import, unicode_literals

# TODO: De Gruyter (publisher)
#
# examples:
# 26110471 Arh Hig Rada Toksikol http://www.degruyter.com/view/j/aiht.2015.66.issue-2/aiht-2015-66-2582/aiht-2015-66-2582.xml
# 12199344: J. Pediatr. Endocrinol. Metab.
# 25390015: Horm Mol Biol Clin Investig
#
# load by dx.doi.org: http://dx.doi.org/10.2478/cdth-2014-0001
#       --> http://www.degruyter.com/view/j/cdth.2014.1.issue-1/cdth-2014-0001/cdth-2014-0001.xml
# PDF -->
# http://www.degruyter.com/dg/viewarticle.fullcontentlink:pdfeventlink/$002fj$002fcdth.2014.1.issue-1$002fcdth-2014-0001$002fcdth-2014-0001.pdf?t:ac=j$002fcdth.2014.1.issue-1$002fcdth-2014-0001$002fcdth-2014-0001.xml

degruyter_journals = ('Arh Hig Rada Toksikol',
                      'Clin Chem Lab Med',
                      'J Pediatr Endocrinol Metab',
                      'Horm Mol Biol Clin Investig',
                      )
