from __future__ import absolute_import, unicode_literals

lancet_journals = {
    'Lancet': {'ja': 'lancet'},
    'Lancet Diabetes Endocrinol': {'ja': 'landia'},
    'Lancet Glob Health': {'ja': 'langlo'},
    'Lancet Haematol': {'ja': 'lanhae'},
    'Lancet HIV': {'ja': 'lanhiv'},
    'Lancet Infect Dis': {'ja': 'laninf'},
    'Lancet Neurol': {'ja': 'laneur'},
    'Lancet Oncol': {'ja': 'lanonc'},
    'Lancet Psychiatry': {'ja': 'lanpsy'},
    'Lancet Respir Med': {'ja': 'lanres'},
}
