from __future__ import absolute_import, unicode_literals

springer_journals = [
        'AAPS PharmSciTech',
        'Acta Clin Croat',
        'Acta Diabetol',
        'Acta Diabetol Lat',
        'Acta Med Croatica',
        'Acta Neurochir (Wien)',
        'Acta Neurochir Suppl',
        'Acta Neurol Belg',
        'Acta Neurol Taiwan',
        'Acta Neuropathol',
        'Adv Exp Med Biol',
        'Adv Hum Genet',
        'Adv Ther',
        'Agents Actions',
        'Aging Clin Exp Res',
        'Albrecht Von Graefes Arch Klin Exp Ophthalmol',
        'Am J Clin Dermatol',
        'Am J Pharmacogenomics',
        'Amino Acids',
        'Anal Bioanal Chem',
        'Angiogenesis',
        'Anim Cogn',
        'Anim Learn Behav',
        'Ann Behav Med',
        'Ann Biomed Eng',
        'Ann Dyslexia',
        'Ann Hematol',
        'Ann Hepatol',
        'Ann N Y Acad Sci',
        'Ann Nucl Med',
        'Ann Surg Oncol',
        'Annu Rev Nurs Res',
        'Antivir Ther (Lond)',
        'Antonie Van Leeuwenhoek',
        'Apoptosis',
        'Appl Biochem Biotechnol',
        'Appl Microbiol Biotechnol',
        'Arch Dermatol Res',
        'Arch Environ Contam Toxicol',
        'Arch Gynecol Obstet',
        'Arch Immunol Ther Exp (Warsz)',
        'Arch Microbiol',
        'Arch Orthop Trauma Surg',
        'Arch Osteoporos',
        'Arch Pharm Res',
        'Arch Sex Behav',
        'Arch Toxicol',
        'Arch Virol',
        'Arch Virol Suppl',
        'Arch Womens Ment Health',
        'Astrophys Space Sci',
        'Atten Defic Hyperact Disord',
        'Basic Life Sci',
        'Basic Res Cardiol',
        'Behav Genet',
        'BioDrugs',
        'Biochem Genet',
        'Biochemistry Mosc',
        'Biodegradation',
        'Biofizika',
        'Biogerontology',
        'Biol Trace Elem Res',
        'Biomech Model Mechanobiol',
        'Biomed Khim',
        'Biomed Microdevices',
        'Biometals',
        'Biomol NMR Assign',
        'Bioorg Khim',
        'Bioprocess Biosyst Eng',
        'Biotechnol Lett',
        'Blut',
        'Brain Nerve',
        'Brain Struct Funct',
        'Brain Topogr',
        'Brain Tumor Pathol',
        'Breast Cancer',
        'Breast Cancer Res Treat',
        'Bull Environ Contam Toxicol',
        'Bull Exp Biol Med',
        'Bull Math Biol',
        'Bundesgesundheitsblatt Gesundheitsforschung Gesundheitsschutz',
        'CNS Drugs',
        'Calcif Tissue Int',
        'Can Anaesth Soc J',
        'Can J Anaesth',
        'Cancer Causes Control',
        'Cancer Chemother Pharmacol',
        'Cancer Immunol Immunother',
        'Cancer Metastasis Rev',
        'Cardiovasc Drugs Ther',
        'Cardiovasc Eng',
        'Cardiovasc Intervent Radiol',
        'Cell Biochem Biophys',
        'Cell Biol Toxicol',
        'Cell Biophys',
        'Cell Mol Life Sci',
        'Cell Mol Neurobiol',
        'Cell Oncol (Dordr)',
        'Cell Stress Chaperones',
        'Cell Tissue Res',
        'Cerebellum',
        'Childs Nerv Syst',
        'Chin J Integr Med',
        'Chirurg',
        'Chromosoma',
        'Chromosome Res',
        'Clin Auton Res',
        'Clin Drug Investig',
        'Clin Exp Med',
        'Clin Exp Metastasis',
        'Clin Exp Nephrol',
        'Clin Investig',
        'Clin Oral Investig',
        'Clin Pharmacokinet',
        'Clin Res Cardiol',
        'Clin Rev Allergy Immunol',
        'Clin Rheumatol',
        'Clin Transl Oncol',
        'Cogn Affect Behav Neurosci',
        'Contemp Rev Obstet Gynaecol',
        'Community Ment Health J',
        'Curr Allergy Asthma Rep',
        'Curr Atheroscler Rep',
        'Curr Diab Rep',
        'Curr Gastroenterol Rep',
        'Curr Genet',
        'Curr Hematol Malig Rep',
        'Curr Hypertens Rep',
        'Curr Microbiol',
        'Curr Oncol Rep',
        'Curr Opin Lipidol',
        'Curr Pain Headache Rep',
        'Curr Psychiatry Rep',
        'Curr Rheumatol Rep',
        'Curr Top Microbiol Immunol',
        'Curr Treat Options Cardiovasc Med',
        'Curr Treat Options Oncol',
        'Cytotechnology',
        'Dermatol Online J',
        'Dev Biol Stand',
        'Dev Genes Evol',
        'Diabetologia',
        'Dig Dis Sci',
        'Doc Ophthalmol',
        'Drug Saf',
        'Drugs',
        'Drugs Aging',
        'Drugs R D',
        'EXS',
        'Ecohealth',
        'Ecotoxicology',
        'Endocr Pathol',
        'Endocr Res',
        'Endocrine',
        'Environ Monit Assess',
        'Environ Sci Pollut Res Int',
        'Eur Arch Otorhinolaryngol',
        'Eur Arch Psychiatry Clin Neurosci',
        'Eur Biophys J',
        'Eur Child Adolesc Psychiatry',
        'Eur J Appl Physiol',
        'Eur J Appl Physiol Occup Physiol',
        'Eur J Clin Microbiol Infect Dis',
        'Eur J Clin Pharmacol',
        'Eur J Drug Metab Pharmacokinet',
        'Eur J Epidemiol',
        'Eur J Health Econ',
        'Eur J Nucl Med',
        'Eur J Nucl Med Mol Imaging',
        'Eur J Nutr',
        'Eur J Pediatr',
        'Eur Phys J E Soft Matter',
        'Eur Radiol',
        'Eur Rev Med Pharmacol Sci',
        'Exp Appl Acarol',
        'Exp Brain Res',
        'Experientia',
        'Extremophiles',
        'Fam Cancer',
        'Fish Physiol Biochem',
        'Fiziol Cheloveka',
        'Folia Microbiol (Praha)',
        'Food Environ Virol',
        'Forensic Sci Med Pathol',
        'Fortschr Chem Org Naturst',
        'Fresenius J Anal Chem',
        'Front Med',
        'Front Med China',
        'Funct Integr Genomics',
        'Gan To Kagaku Ryoho',
        'Gastric Cancer',
        'Gen Thorac Cardiovasc Surg',
        'Genetica',
        'Genetika',
        'Glycoconj J',
        'Graefes Arch Clin Exp Ophthalmol',
        'HNO',
        'Haematol Blood Transfus',
        'Hautarzt',
        'Heart Vessels',
        'Hepatol Int',
        'Hernia',
        'Herz',
        'Histochem Cell Biol',
        'Histochem J',
        'Histochemistry',
        'Horm Cancer',
        'Humangenetik',
        'Hum Genet',
        'Immunogenetics',
        'Immunol Res',
        'In Vitro',
        'In Vitro Cell Dev Biol',
        'In Vitro Cell Dev Biol Anim',
        'Indian J Gastroenterol',
        'Indian J Pediatr',
        'Indian Pediatr',
        'Infection',
        'Inflamm Res',
        'Inflammation',
        'Int Arch Occup Environ Health',
        'Int J Biometeorol',
        'Int J Cardiovasc Imaging',
        'Int J Clin Lab Res',
        'Int J Clin Monit Comput',
        'Int J Clin Oncol',
        'Int J Clin Pharm',
        'Int J Colorectal Dis',
        'Int J Comput Assist Radiol Surg',
        'Int J Gastrointest Cancer',
        'Int J Hematol',
        'Int J Legal Med',
        'Int J Pancreatol',
        'Int Ophthalmol',
        'Int Orthop',
        'Int Urogynecol J',
        'Int Urogynecol J Pelvic Floor Dysfunct',
        'Int Urol Nephrol',
        'Internist (Berl)',
        'Intensive Care Med',
        'Interdiscip Sci',
        'Intern Emerg Med',
        'Invert Neurosci',
        'Invest New Drugs',
        'Ir J Med Sci',
        'J Abnorm Child Psychol',
        'J Am Soc Mass Spectrom',
        'J Anesth',
        'J Appl Genet',
        'J Assist Reprod Genet',
        'J Autism Dev Disord',
        'J Behav Med',
        'J Bioenerg Biomembr',
        'J Biol Inorg Chem',
        'J Biomed Sci',
        'J Biomol NMR',
        'J Biosci',
        'J Bone Miner Metab',
        'J Cancer Educ',
        'J Cancer Res Clin Oncol',
        'J Cardiovasc Transl Res',
        'J Chem Ecol',
        'J Clin Immunol',
        'J Comp Physiol A Neuroethol Sens Neural Behav Physiol',
        'J Comp Physiol B, Biochem Syst Environ Physiol',
        'J Comput Aided Mol Des',
        'J Endocrinol Invest',
        'J Fluoresc',
        'J Gambl Stud',
        'J Gastroenterol',
        'J Gastrointest Cancer',
        'J Gastrointest Surg',
        'J Genet',
        'J Genet Couns',
        'J Hepatobiliary Pancreat Surg',
        'J Huazhong Univ Sci Technol Med Sci',
        'J Ind Microbiol',
        'J Ind Microbiol Biotechnol',
        'J Infect Chemother',
        'J Inherit Metab Dis',
        'J Interv Card Electrophysiol',
        'J La State Med Soc',
        'J Mater Sci Mater Med',
        'J Math Biol',
        'J Membr Biol',
        'J Microbiol',
        'J Mol Evol',
        'J Mol Histol',
        'J Mol Med',
        'J Mol Med (Berl)',
        'J Mol Model',
        'J Mol Neurosci',
        'J Muscle Res Cell Motil',
        'J Nephrol',
        'J Neural Transm',
        'J Neural Transm Gen Sect',
        'J Neural Transm Suppl',
        'J Neurocytol',
        'J Neuroimmune Pharmacol',
        'J Neurol',
        'J Neurooncol',
        'J Neurovirol',
        'J Nucl Cardiol',
        'J Nucl Med',
        'J Nutr Health Aging',
        'J Orthop Sci',
        'J Physiol Biochem',
        'J Physiol Sci',
        'J Plant Res',
        'J Protein Chem',
        'J Relig Health',
        'J Thromb Thrombolysis',
        'J Tongji Med Univ',
        'JIMD Rep',
        'Jpn J Ophthalmol',
        'Jpn J Radiol',
        'Klin Wochenschr',
        'Knee Surg Sports Traumatol Arthrosc',
        'Langenbecks Arch Surg',
        'Lasers Med Sci',
        'Learn Behav',
        'Lipids',
        'Lung',
        'MAGMA',
        'Mamm Genome',
        'Mar Biotechnol',
        'Matern Child Health J',
        'Med Biol Eng Comput',
        'Med Klin (Munich)',
        'Med Lett Drugs Ther',
        'Med Microbiol Immunol',
        'Med Mol Morphol',
        'Med Oncol',
        'Med Tekh',
        'Mem Cognit',
        'Metab Brain Dis',
        'Methods Mol Biol',
        'Methods Mol Med',
        'Microb Ecol',
        'Mikrobiologiia',
        'Minerva Endocrinol',
        'Mod Rheumatol',
        'Mol Biol (Mosk)',
        'Mol Biol Med',
        'Mol Biol Rep',
        'Mol Biotechnol',
        'Mol Cell Biochem',
        'Mol Diagn',
        'Mol Diagn Ther',
        'Mol Divers',
        'Mol Gen Genet',
        'Mol Gen Mikrobiol Virusol',
        'Mol Genet Genomics',
        'Mol Imaging Biol',
        'Mol Neurobiol',
        'Mycopathologia',
        'Mycorrhiza',
        'Mycotoxin Res',
        'Naunyn Schmiedebergs Arch Pharmacol',
        'Ned Tijdschr Geneeskd',
        'Nervenarzt',
        'Neurochem Res',
        'Neurogenetics',
        'Neurol Med Chir (Tokyo)',
        'Neurol Sci',
        'Neuromolecular Med',
        'Neuroradiology',
        'Neurosci Bull',
        'Neurosurg Rev',
        'Neurotox Res',
        'No To Hattatsu',
        'Obes Surg',
        'Odontology',
        'Ophthalmologe',
        'Oral Maxillofac Surg',
        'Orig Life Evol Biosph',
        'Orthopade',
        'Osteoporos Int',
        'Paediatr Drugs',
        'Parasitol Res',
        'Pathol Oncol Res',
        'Pathologe',
        'Pediatr Cardiol',
        'Pediatr Dev Pathol',
        'Pediatr Nephrol',
        'Pediatr Radiol',
        'Pediatr Surg Int',
        'Percept Psychophys',
        'Pflugers Arch',
        'Pharm Res',
        'Pharm Weekbl Sci',
        'Pharm World Sci',
        'Pharmacoeconomics',
        'Photosyn Res',
        'Pituitary',
        'Plant Cell Rep',
        'Plant Mol Biol',
        'Planta',
        'Prikl Biokhim Mikrobiol',
        'Protein Cell',
        'Protein J',
        'Protoplasma',
        'Psychol Res',
        'Psychopharmacologia',
        'Psychopharmacology (Berl)',
        'Radiat Environ Biophys',
        'Radiologe',
        'Recent Prog Horm Res',
        'Recent Results Cancer Res',
        'Results Probl Cell Differ',
        'Rev Endocr Metab Disord',
        'Rev Physiol Biochem Pharmacol',
        'Rheumatol Int',
        'Rinsho Ketsueki',
        'Rom J Morphol Embryol',
        'Schizophr Res',
        'Schmerz',
        'Sci China, C, Life Sci',
        'Sci China Life Sci',
        'Sex Plant Reprod',
        'Skeletal Radiol',
        'Sleep Breath',
        'Somat Cell Mol Genet',
        'Somatic Cell Genet',
        'Sports Med',
        'Stem Cell Rev',
        'Strahlenther Onkol',
        'Subcell Biochem',
        'Support Care Cancer',
        'Surg Endosc',
        'Surg Today',
        'Target Oncol',
        'Theor Appl Genet',
        'Theory Biosci',
        'Transgenic Res',
        'Transpl Int',
        'Trop Anim Health Prod',
        'Tsitol Genet',
        'Tumori',
        'Tumour Biol',
        'Unfallchirurg',
        'Updates Surg',
        'Urol Res',
        'Urologe A',
        'Vet Res Commun',
        'Virchows Arch',
        'Virchows Arch A Pathol Anat Histol',
        'Virol Sin',
        'Virus Genes',
        'Vopr Onkol',
        'Wien Klin Wochenschr',
        'World J Microbiol Biotechnol',
        'World J Pediatr',
        'World J Surg',
        'World J Urol',
        'Yao Xue Xue Bao',
        'Z Gerontol Geriatr',
        'Z Kardiol',
        'Z Parasitenkd',
        'Z Rheumatol',
        'Zh Evol Biokhim Fiziol',
        'Zhongguo Yi Xue Ke Xue Yuan Xue Bao',
        'Zhonghua Fu Chan Ke Za Zhi',
        'Zhonghua Xue Ye Xue Za Zhi',
]

