from __future__ import absolute_import, unicode_literals

# Wolters Kluwer
#
# Some of this stuff is Free and/or Open Access. 
#
# http://journals.lww.com/pain/Fulltext/2015/07000/Social_communication_model_of_pain.7.aspx

wolterskluwer_journals = [
    'Acad Med',
    'Adv Anat Pathol',
    'AIDS',
    'Alzheimer Dis Assoc Disord',
    'Am J Dermatopathol',
    'Am J Forensic Med Pathol',
    'Am J Ther',
    'Am J Clin Oncol',
    'Am J Med Sci',
    'Am J Physiol Heart Circ Physiol',
    'Am J Surg Pathol',
    'Anesth Analg',
    'Ann Plast Surg',
    'Ann Surg',
    'Anticancer Drugs',
    'Appl Immunohistochem Mol Morphol',
    'ASAIO J',
    'Behav Pharmacol',
    'Blood Coagul Fibrinolysis',
    'Cancer J',
    'Clin Dysmorphol',
    'Clin J Pain',
    'Clin J Sport Med',
    'Clin Neuropharmacol',
    'Clin Obstet Gynecol',
    'Clin Orthop Relat Res',
    'Cornea',
    'Coron Artery Dis',
    'Crit Care Med',
    'Curr Opin Allergy Clin Immunol',
    'Curr Opin Cardiol',
    'Curr Opin Clin Nutr Metab Care',
    'Curr Opin Crit Care',
    'Curr Opin Hematol',
    'Curr Opin HIV AIDS',
    'Curr Opin Infect Dis',
    'Curr Opin Nephrol Hypertens',
    'Curr Opin Obstet Gynecol',
    'Curr Opin Oncol',
    'Curr Opin Ophthalmol',
    'Curr Opin Pulm Med',
    'Curr Opin Rheumatol',
    'Curr Opin Urol',
    'Dermatitis',
    'Diagn Mol Pathol',
    'Dis Colon Rectum',
    'Ear Hear',
    'Epidemiology',
    'Eur J Anaesthesiol',
    'Eur J Cancer Prev',
    'Eur J Gastroenterol Hepatol',
    'Eye Contact Lens',
    'Fam Community Health',
    'Health Phys',
    'Implant Dent',
    'Inflamm Bowel Dis',
    'Int Clin Psychopharmacol',
    'Int J Gynecol Pathol',
    'Invest Radiol',
    'J Acquir Immune Defic Syndr',
    'J Acquir Immune Defic Syndr Hum Retrovirol',
    'J Addict Med',
    'J Burn Care Res',
    'J Cardiovasc Med (Hagerstown)',
    'J Cardiovasc Pharmacol',
    'J Cereb Blood Flow Metab',
    'J Clin Neuromuscul Dis',
    'J Clin Neurophysiol',
    'J Clin Rheumatol',
    'J Comput Assist Tomogr',
    'J Craniofac Surg',
    'J Dev Behav Pediatr',
    'J Glaucoma',
    'J Head Trauma Rehabil',
    'J HIV Ther',
    'J Hypertens Suppl',
    'J Hypertens',
    'J Immunother',
    'J Investig Med',
    'J Neuroophthalmol',
    'J Neuropathol Exp Neurol',
    'J Neurosurg Anesthesiol',
    'J Nurs Res',
    'J Occup Environ Med',
    'J Pediatr Hematol Oncol',
    'J Pediatr Gastroenterol Nutr',
    'J Pediatr Orthop',
    'J Pediatr Orthop B',
    'J Perinat Neonatal Nurs',
    'J Psychiatr Pract',
    'J Spinal Disord Tech',
    'J Strength Cond Res',
    'J Thorac Oncol',
    'J Trauma',
    'MCN Am J Matern Child Nurs',
    'Med Care',
    'Med Sci Sports Exerc',
    'Medicine (Baltimore)',
    'Melanoma Res',
    'Menopause',
    'Neurologist',
    'Neuroreport',
    'Neurosurgery',
    'Nucl Med Commun',
    'Obstet Gynecol',
    'Obstet Gynecol Surv',
    'Ophthal Plast Reconstr Surg',
    'Optom Vis Sci',
    'Otol Neurotol',
    'Pain',
    'Pathology'
    'Pediatr Crit Care Med',
    'Pediatr Emerg Care',
    'Pediatr Infect Dis J',
    'Pharmacogenet Genomics',
    'Pharmacogenetics',
    'Plast Reconstr Surg',
    'Psychiatr Genet',
    'Psychosom Med',
    'Retina (Philadelphia, Pa)',
    'Retin Cases Brief Rep',
    'Sex Transm Dis',
    'Shock',
    'South Med J',
    'Spine',
    'Ther Drug Monit',
    'Transplantation',
    'Zhonghua Yu Fang Yi Xue Za Zhi',
]

